import java.io.FileWriter
/**Provides multiple ways of saving comments in human readable format.
 * Can link to other sources in Src/ but should be able to work just as plain .html (except for the video of course)*/
object HtmlWriter {
    /**Writes comments to html without making them "chained" (if you can even call youtubes system like that)
     * all of them are linked by [Comment.cid] so you can navigate between replies like on standard image board*/
    fun basicLinkedComments(comments :Array<Comment>) {
        val writer = FileWriter(Config.outputFile)
        val meta = MetaData(Config.videoToLink, "", "", "", 0, 0)
        writeHead(writer, meta)
        writeVideoPart(writer, Config.videoToLink)
        writer.write("<ul>\n")
        comments.forEach {
            writeSingleComment(it, writer)
        }
        writer.write("</ul>\n")
        finalizeHtml(writer)
        writer.close()
    }

    /**Attempts to write comments in a way that looks as close to youtubes style of chaining as possible*/
    fun youtubeStyleComments() {

    }

    //=====================\\
    /**Writes HTML header and possibly metadata?*/
    private fun writeHead(writer :FileWriter, meta :MetaData) {
        val header = "<!DOCTYPE html>\n" +
                "<html>\n" +
                "  <head>\n" + //Todo: Add meta info to html comments? - could be usefull in case of loss of meta.json
                "    <meta charset=\"UTF-8\">\n" +
                "    <title>" + meta.videoName + "</title>" +
                "  </head>\n" +
                "  <body>"
        writer.write(header)
    }

    private fun writeVideoPart(writer :FileWriter, videoLink :String) {
        val videoPart =
                "    <video controls>\n" +
                "      <source src=\"" + videoLink + "\" type=\"" + Utils.figureOutVideoMime(videoLink) + "\"/>\n" +
                "      Your browser does not support video\n" +
                "      </video>" //Add support for multiple sources?
                                //Todo: add thubnail support
        writer.write(videoPart)
    }

    /**Used for basic comment writing, this function does not care about structure and just writes simple < li >*/
    private fun writeSingleComment(comment: Comment, writer: FileWriter) {

        val link = comment.cid?.let { //todo: migrate this to Comment.getCID
            if (comment.isCommentReply()) {
                //In case this is reply the second part of cid is this comments actual id
                return@let it.split(".")[1]
            } else {
                return@let it
            }
        }
        writer.write("<li id=\"$link\">\n") //musíme zjistit zda je komment odpověď na něco
        //todo: write anvatar in future
        //potřebujem: autora text, link na comment kam odpovídá
        writer.write("<p class=\"author\"> ${comment.author} ($link)</p>\n")
        if (comment.isCommentReply()) {
            val replyTo = comment.cid?.let { return@let it.split(".")[0]}
            writer.write("<p class=\"reply\"><a href=\"#$replyTo\">>>$replyTo</a></p>\n")
        }
        writer.write("<p class=\"comment-text\"> ${comment.text}</p>\n")
        writer.write("</li>\n")
    }

    private fun finalizeHtml(writer :FileWriter) {
        writer.write("  </body>\n</html>\n")
        writer.flush()
    }
}