import java.io.*
import java.net.HttpURLConnection
import java.net.URL
import java.nio.ByteBuffer

object Utils {
    /*
    * If changing Dir paths:
    * 1) Paths MUST be relative in order to work properly when unzipping
    * 2) If you something should be in root use just "." or "./"
    * 3) Doesnt matter they do nothing
    */
    //Maybe everything should be lowercase
    /**For meta files? right now we have probably just one*/
    const val META_DIR :String = "Meta/"
    /**Could contain RAW video, JSON comments maybe*/
    const val RAW_DIR :String = "RAW/"
    /**Could contain .CSS, .JS, and other source files for main HTML file*/
    const val SOURCES_DIR :String = "Src/"
    /**Porbably good idea to save possibly hunderds of files to sub dir*/
    const val AVATARS_DIR :String = "Src/Avatars/"

    /**Converts broken JSON produced by comment downloader to valid JSON
     * @param oldFile Path to original filepath
     * @param newFile Path where converted JSON will be saved to*/
    fun fixJson(oldFile :String, newFile :String) {
        val reader = BufferedReader(FileReader(File(oldFile)))
        val writer = FileWriter(newFile, false);
        var line :String?

        writer.write("[")
        line = reader.readLine()
        line?.let {
            if (it.startsWith("[")) {
                println("File is most probably already in desired format, or it's not broken by standart comment downloader")
                println("Exiting...")
                return
            }
        }
        while (line != null) {
            writer.write(line)
            line = reader.readLine()
            line?.let { writer.write(",\n") }
        }
        writer.write("\n]\n")
        writer.flush()
        println("Done!")
    }

    /**Downloads all avatars, BUT DOES NOT CHECK FOR DUPLICATES, THEY WILL GET AUTOMATICALLY OVERWRITED.
     * [customAvatarDir] is broken and does nothing*/
    fun downloadAllAvatars(from : Array<Comment>, customAvatarDir :String = AVATARS_DIR) {
        if (!from[0].isNewFormat()) {
            println("Error: Can't download avatars from file that has no link to them!")
            return
        }

        from.forEach { com ->
            com.photo?.let {
                downloadFile(it, getFileName(it))
                println("Downloading: $it to ${getFileName(it)}")
                Thread.sleep(300) //bigger or smaller?
            }
        }
    }

    fun figureOutVideoMime(file :String) :String {
        if (file.endsWith(".mp4")) return "video/mp4"
        if (file.endsWith(".webm")) return "video/webm"
        if (file.endsWith(".avi")) return "video/avi"
        if (file.endsWith(".ogg") || file.endsWith(".ogv")) return "video/ogg"
        return "video/mpeg" //I don't know at this point so just try something, shouldn't happen anyway
    }

    private fun getFileName(link :String) :String {
        return link.substring(link.lastIndexOf("/", link.length)+1, link.length)
    }

    private fun downloadFile(url :String, toFile :String) {
        val link = URL(url)
        try {
            val con :HttpURLConnection= link.openConnection() as HttpURLConnection
            con.doOutput = true

            //Probably could use more error checks
            if (con.responseCode != HttpURLConnection.HTTP_NOT_FOUND) {
                val writer = FileOutputStream(toFile)
                val reader = con.inputStream
                val buffer = ByteBuffer.allocate(2048).array()

                var count = reader.read(buffer)
                while (count > 0) {
                    writer.write(buffer, 0, count)
                    count = reader.read(buffer)
                }
                reader.close()
                writer.flush()
                writer.close()
            }
        } catch (e :Exception) {
            System.err.println("Error while downloading avatar: $e")
        }
    }

}